const os = require('os');
const path = require('path');
const fs = require('fs-extra');
const axios = require('axios');
const { spawn } = require('child_process');
const { prompt, Select, Toggle } = require('enquirer');

function Core(target) {
    const configuration = fs.readJsonSync(target, { throws: false });
    
    let ptr = null;
    let ready = true;

    this.configuration = configuration;
    this.target = target;
    this.api = `https://tanker.xarlike.info/api/`;
    this.timeout = 1000;

    if (configuration && configuration.api && configuration.timeout && configuration.token) {
        const { api: baseURL, timeout, token } = configuration;
        const params = {
            baseURL,
            timeout,
            headers: { 'Authorization': `Bearer ${token}` }
        };
        ptr = axios.create(params);
    }
    this.axios = ptr;
}

Core.prototype.before = function (message) {
    const { axios } = this;
    if (axios === null) {
        throw new Error(message);
    }
};

Core.prototype.error = function (message) {
    console.error(message);
};

Core.prototype.debug = async function () {
    const { configuration } = this;
    console.log("configuration", configuration);
};

Core.prototype.random = function (len) {
    return [...Array(12)].map(i => (~~(Math.random() * len)).toString(len)).join('');
};

Core.prototype.configure = async function () {
    const { api, timeout } = this;

    try {
        const token = await prompt({
            type: 'input',
            name: 'token',
            message: 'What is your token ?'
        });

        const state = Object.assign({}, token, { api, timeout });

        fs.writeJsonSync(this.target, state);
        console.log('Token added, the next step is to run this command "tanker-cli use"');
    } catch (e) {}
};

Core.prototype.reset = async function () {
    const { target } = this;

    try {
        this.before('There is no configuration to delete.');
        const prompt = new Toggle({
            message: 'Do you can to remove your access token ?',
            enabled: 'Yes',
            disabled: 'No'
        });
        prompt.run()
            .then(answer => {
                answer && fs.writeJsonSync(target, {})
            })
            .catch((e) => {});
    } catch (e) {
        this.error(e.message);
    }
};

Core.prototype.use = async function () {
    const { configuration, axios, target } = this;
    const message = 'Need to run this command "tanker-cli configure" first';
    try {
        this.before(message);

        const { data } = await axios.get("/codes");

        const choices = data.map((el) => {
            return { name: el.title, value: el.id };
        });

        if (choices.length === 0) {
            return console.error('Nothing found');
        }

        const prompt = new Select({
            message: 'Which script do you want to select ?',
            choices,
            result(names) {
                return this.map(names);
            }
        });

        prompt.run()
            .then(res => {
                const keys = Object.keys(res);
                keys.forEach(async (key) => {
                    const codeId = res[key];
                    const { data } = await axios.get(`/codes/${codeId}`);
                    const state = Object.assign({}, configuration, { payload: data });
                    fs.writeJsonSync(target, state)
                });
            })
            .catch((e) => { });
    } catch (e) {
        if (e?.response?.status === 401) {
            console.error("401 - Unauthorized");
            return console.error(message);
        }
        console.log(e.message);
    }
};

Core.prototype.run = async function ({ bin }) {
    const { configuration } = this;

    const once = this.random(36);
    const fileName = path.join(os.tmpdir(), `payload-${once}.sh`);

    try {
        if (configuration?.payload?.content) {
            fs.writeFileSync(fileName, configuration.payload.content, { mode: 0o500 });
        } else {
            return this.error('Need to run this command "tanker-cli use" first');
        }

        const args = [...process.argv];
        while (args.shift() !== "run") { }
        args.unshift(fileName);
        const bat = spawn(bin, args);
        bat.stdout.on('data', (data) => {
            process.stdout.write(data.toString());
        });
        bat.stderr.on('data', (data) => {
            process.stderr.write(data.toString());
        });
    } catch (e) {
        return console.error(e);
    }
};

Core.prototype.display = async function () {
    const { configuration } = this;
    if (configuration?.payload?.content) {
        return process.stdout.write(configuration.payload.content);
    } else {
        return this.error('Need to run this command "tanker-cli use" first');
    }
};

Core.prototype.status = async function () {
    const { axios } = this;

    const message = 'Need to run this command "tanker-cli configure" first';
    try {
        this.before(message);

        const { data } = await axios.get(`/user`);
        console.log("200 - Access Granted");
        console.log();
        console.log(`name: ${data.name}`);
        console.log(`email: ${data.email}`);
    } catch (e) {
        if (e?.response?.status === 401) {
            this.error("401 - Unauthorized");
            return this.error(message);
        }
        console.log(e.message);
    }
};

const generate = () => {
    const filename = '.tanker';
    const target = path.join(process.env.HOME, filename);

    const core = new Core(target);
    core.Core = Core;
    return core;
};

module.exports = generate();